from unittest import TestCase

from k8s_handle.k8s.mocks import ResourcesAPIMock

from .resource_getters import CoreResourceGetter, RegularResourceGetter
from .mocks import MockResource


class TestCoreResourceGetter(TestCase):

    def setUp(self):
        self.getter = CoreResourceGetter(
            ResourcesAPIMock(group_version="v1", resources=[MockResource("Pod"), MockResource("CronJob")])
        )

    def test_is_processable_version(self):
        self.assertTrue(self.getter.is_processable_version("v1"))
        self.assertFalse(self.getter.is_processable_version("app/v1"))
        self.assertFalse(self.getter.is_processable_version("/"))
        self.assertFalse(self.getter.is_processable_version(""))

    def test_get_resources_by_version(self):
        self.assertSetEqual({"Pod", "CronJob"}, self.getter.get_resources_by_version("v1"))
        self.assertSetEqual(set(), self.getter.get_resources_by_version("v2"))


class TestRegularResourceGetter(TestCase):

    def setUp(self):
        self.getter = RegularResourceGetter(
            ResourcesAPIMock(group_version="app/v1", resources=[MockResource("Pod"), MockResource("CronJob")])
        )

    def test_is_processable_version(self):
        self.assertFalse(self.getter.is_processable_version("v1"))
        self.assertTrue(self.getter.is_processable_version("app/betav1"))
        self.assertTrue(self.getter.is_processable_version("app/v1"))
        self.assertFalse(self.getter.is_processable_version("/"))
        self.assertFalse(self.getter.is_processable_version(""))

    def test_get_resources_by_version(self):
        self.assertSetEqual({"Pod", "CronJob"}, self.getter.get_resources_by_version("app/v1"))
        self.assertSetEqual(set(), self.getter.get_resources_by_version("app/betav1"))



class ProvisioningError(Exception):
    pass


class DeprecationError(Exception):
    pass


class ResourceNotAvailableError(Exception):
    pass


class InvalidYamlError(Exception):
    pass


class TemplateRenderingError(Exception):
    pass



from unittest import TestCase

from .checker import ResourceAvailabilityChecker
from .resource_getters import CoreResourceGetter, RegularResourceGetter
from .mocks import MockResource

from k8s_handle.k8s.mocks import ResourcesAPIMock
from k8s_handle.exceptions import ResourceNotAvailableError


class TestAvailabilityChecker(TestCase):

    def setUp(self):
        core_getter = CoreResourceGetter(
            ResourcesAPIMock(group_version="v1", resources=[MockResource("Pod"), MockResource("CronJob")])
        )

        regular_getter = RegularResourceGetter(
            ResourcesAPIMock(group_version="app/v1", resources=[MockResource("Deployment"), MockResource("Service")])
        )

        self.checker = ResourceAvailabilityChecker([core_getter, regular_getter])

    def test_is_available_kind(self):
        self.assertTrue(self.checker._is_available_kind("v1", "Pod"))
        self.assertTrue(self.checker._is_available_kind("app/v1", "Service"))
        self.assertFalse(self.checker._is_available_kind("v1", "Deployment"))
        self.assertFalse(self.checker._is_available_kind("app/v1", "CronJob"))

    def test_run_with_valid_version(self):
        self.checker.run('k8s_handle/k8s/fixtures/valid_version.yaml')

    def test_run_with_invalid_version(self):
        with self.assertRaises(ResourceNotAvailableError):
            self.checker.run('k8s_handle/k8s/fixtures/invalid_version.yaml')

    def test_run_with_unsupported_version(self):
        with self.assertRaises(ResourceNotAvailableError):
            self.checker.run('k8s_handle/k8s/fixtures/unsupported_version.yaml')



import re
from typing import List, Set

from k8s_handle.k8s.api_extensions import ResourcesAPI, CoreResourcesAPI

core_pattern = re.compile(r'^([^/]+)$')
regular_pattern = re.compile(r'^([^/]+)/([^/]+)$')


class AbstractResourceGetter(object):
    def is_processable_version(self, api_group: str) -> bool:
        raise NotImplementedError

    def get_resources_by_version(self, api_group: str) -> Set[str]:
        raise NotImplementedError


class CoreResourceGetter(AbstractResourceGetter):

    def __init__(self, resource_api: CoreResourcesAPI):
        self.api = resource_api
        self.versions = {}

    def is_processable_version(self, api_group: str) -> bool:
        return core_pattern.match(api_group) is not None

    def get_resources_by_version(self, api_group: str) -> Set[str]:
        if api_group in self.versions:
            return self.versions[api_group]

        resp = self.api.list_api_resources(api_group)

        if not resp:
            return set()

        kinds = {r.kind for r in resp.resources}
        self.versions[api_group] = kinds

        return kinds


class RegularResourceGetter(AbstractResourceGetter):

    def __init__(self, resource_api: ResourcesAPI):
        self.api = resource_api
        self.versions = {}

    def is_processable_version(self, api_group: str) -> bool:
        return regular_pattern.match(api_group) is not None

    def get_resources_by_version(self, api_group: str) -> Set[str]:
        if api_group in self.versions:
            return self.versions[api_group]

        group, version = regular_pattern.findall(api_group).pop()
        resp = self.api.list_api_resource_arbitrary(group, version)

        if not resp:
            return set()

        kinds = {r.kind for r in resp.resources}
        self.versions[api_group] = kinds

        return kinds


def make_resource_getters_list() -> List[AbstractResourceGetter]:
    return [
        CoreResourceGetter(CoreResourcesAPI()),
        RegularResourceGetter(ResourcesAPI())
    ]
